/*
    Copyright (C) 2019 Minibot
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Minibot nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#include "NanopixCore.h"

using namespace NanopixCore; 

MainController & nanopix = MainController::getInstance();

void setup() { }

// ------------------------------------- flightScript ---------------------------------------------
// ------- English: Write your own program for Nanopix inside the flightScript() function.
// ------- In this example we calculate accelerometer X axis measurement variance and report it.
// ------------------------------------------------------------------------------------------------
// ------- Russian: Напишите свою программу для Nanopix внутри функции flightScript().
// ------- В текущем примере рассчитывается среднее среднеквадратическое отклонение измерения 
// ------- акселерометра по оси Х.
// ------------------------------------------------------------------------------------------------
void flightScript()
{
   long next_report_ms = millis() + 1000;
   float average_accel_x = 0;
   float summ_accel_x = 0;
   float summ_var_accel_x = 0;
   float variance_accel_x = 0;
   long counter = 0;
   float value;
   while(true) 
   {
     // English: get normilized axis X accelerometer measurement (1.0f ~ 1 g)
     // Russian: получаем нормализованное значение измерения акселерометра по оси Х
     value = nanopix.getAccelX();

     // English: summ to calculate 1 second average value
     // Russian: суммируем для получения среднего значения за период в 1 сек
     summ_accel_x += value;
     summ_var_accel_x += (value - average_accel_x) * (value - average_accel_x);
     counter++;

     // English: each second we calculate average values and report to Nanopix Ground Station
     // Russian: каждую секунду расчитываем и отпавляем результат в лог наземной станции
     if(millis() > next_report_ms)
     {
       average_accel_x = summ_accel_x / counter;
       variance_accel_x = sqrt(summ_var_accel_x / counter);
       nanopix.sendLogMsg(variance_accel_x);
       summ_var_accel_x = 0;
       summ_accel_x = 0;
       counter = 0;
       next_report_ms += 1000;
     }
     delay(20);
   }
}
// ------------------------------------------------------------------------------------------------

void loop() {
  nanopix.mainRoutine();
  
  if(nanopix.isMissionTriggered())
  {
    delay(1000);
    flightScript();
    nanopix.isArmCommandReceived();
	  nanopix.resetMissionTrigger();
  }
}
