/*
    Copyright (C) 2019 Minibot
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Minibot nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#include "NanopixCore.h"

using namespace NanopixCore; 

MainController & nanopix = MainController::getInstance();

void setup() { }

// ------------------------------------- flightScript ---------------------------------------------
// ------- English: Write your own program for Nanopix inside the flightScript() function.
// ------- In this example Nanopix takeoff, flies 0.6 meters forward, next move 1.2 meters diameter circle with 
// ------- counterclockwise direction.
// ------- (!) Prepare free flight zone for such movement.
// ------------------------------------------------------------------------------------------------
// ------- Russian: В текущем примере дрон взлетает, пролетает вперед и затем двигается по окружности 
// ------- по часовой стрелке один оборот, затем возвращается в исходную позицию и садится. 
// ------- Диаметр окружности 1.2 метра.
// ------- (!) Предусмотрите свободное место для такого маневра!
// ------------------------------------------------------------------------------------------------
void flightScript()
{
   // English: Automatic takeoff up to 60 cm. Blocking function.
   // Russian: Автовзлет, эта функция выполняется до того момента как дрон поднимется 
   // и зафиксирует высоту в 60 сантиметров.
   nanopix.takeoff();

   float start_x = nanopix.getPosX();
   float start_y = nanopix.getPosY();

   // English: move 0.6 meters forward
   // Russian: пролёт вперед на 60 см
   nanopix.offsetTargetPos(0, 0.6f); 
    
   for(int i = 0; i <= 360; i++)
   {
    // English: compute coordinates offset
    // Russian: расчитываем смещение координат относительно центра окружности
    float dx = 0.6f * sin(i * M_PI / 180.0f);
    float dy = 0.6f * cos(i * M_PI / 180.0f);

    // English: do not waiting all points, except last one
    // Russian: без ожидания достижения координат для всех итераций кроме последней 
    bool is_wait = (i == 360); 

    // English: target point moving in a circle with center in (start_x, start_y)
    // Russian: точка целевой позиции движется по окружности с центром в (start_x, start_y)
    nanopix.setTargetPos(start_x + dx, start_y + dy, is_wait); 
    delay(60);
   }
   delay(500);

   // English: move 0.6 meters backward
   // Russian: пролёт назад на 60 см
   nanopix.offsetTargetPos(0, -0.6f);
  
   nanopix.land();
}
// ------------------------------------------------------------------------------------------------

void loop() {
  nanopix.mainRoutine();
  
  if(nanopix.isMissionTriggered())
  {
    delay(1000);
    flightScript();
	  nanopix.resetMissionTrigger();
  }
}
