#!/usr/bin/env python
# -*- coding: utf-8 -*-
import json
import os
import socket
import subprocess
import sys
import argparse
from shutil import copyfile, move
from concurrent.futures import ThreadPoolExecutor
from sys import platform

def remove_ino_files(dir_path):
    files = os.listdir(dir_path)
    for item in files:
        if item.endswith(".ino"):
            os.remove(os.path.join(dir_path, item))


def print_popen_pipe(p, stdfile):

    while p.poll() is None:
        line = stdfile.readline()
        if line != b'':
            print(line)
            sys.stdout.flush()

    # Write the rest from the buffer
    tail = stdfile.read()
    if tail != b'':
        print(tail)
        sys.stdout.flush()

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument('-i', type=str, dest='server_ip',
                        help='upload server IP address', required=True)
    parser.add_argument('-p', type=str, dest='server_port',
                        help='upload server port', required=True)
    parser.add_argument('-c', type=str, dest='chip',
                        help='chip id', required=True)
    parser.add_argument('-f', type=str, dest='ino_path',
                        help='path to ino file')
    parser.add_argument('-b', type=str, dest='board_id',
                        help='board ID', required=True)
    parser.add_argument('-t', type=str, dest='target_operation',
                        help='target operation (build or upload, or all)')

    args = parser.parse_args()

    ino_path = args.ino_path
    chip_id = args.chip

    APP_HOST = args.server_ip
    APP_PORT = int(args.server_port)

    board_id = args.board_id
    if "NANOPIX_V8" in board_id:
        env_name = "nanopix-v8"
    if "NANOPIX_V7" in board_id:
        env_name = "nanopix-v7"

    build = True
    upload = True
    target_operation = args.target_operation
    if target_operation is not None:
        if target_operation == "build":
            upload = False
        if target_operation == "upload":
            build = False


    pio_path = os.path.join(os.path.dirname(sys.executable), "Scripts", "pio-script.py")
    esp_core_lib_dir = os.path.dirname(os.path.realpath(__file__))
    package_base_dir = os.path.dirname(os.path.dirname(esp_core_lib_dir))
    tools_dir = os.path.join(os.path.dirname(os.path.dirname(os.path.dirname(package_base_dir))), "tools")
    platform_path = os.path.join(esp_core_lib_dir, "espressif8266-1.2.1-custom.zip")
    roboton_path = os.path.dirname(esp_core_lib_dir)

    if build:
        try:
            move(os.path.join(tools_dir, "xtensa", "1.20.0-26-gb404fb9-2", "1.20.0-26-gb404fb9-2.zip"),
                 esp_core_lib_dir)
        except Exception as e:
            print(e)


        if platform == "linux" or platform == "linux2":
            exec_list = ["pio"]
        elif platform == "win32":
            exec_list = [sys.executable, pio_path]
        with subprocess.Popen(exec_list + ["platform", "install",
                                 "file://" + platform_path],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE) as p:
            with ThreadPoolExecutor(2) as pool:
                r1 = pool.submit(print_popen_pipe, p, p.stdout)
                r2 = pool.submit(print_popen_pipe, p, p.stderr)
                r1.result()
                r2.result()


        src_dir = os.path.join(esp_core_lib_dir, "src")
        remove_ino_files(src_dir)
        copyfile(ino_path, os.path.join(src_dir, "sketch.ino"))


        with subprocess.Popen(exec_list + ["run", "-e", env_name, "-d",
                                 esp_core_lib_dir],
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE) as p:
            with ThreadPoolExecutor(2) as pool:
                r1 = pool.submit(print_popen_pipe, p, p.stdout)
                r2 = pool.submit(print_popen_pipe, p, p.stderr)
                r1.result()
                r2.result()

    try:
        move(os.path.join(esp_core_lib_dir, "1.20.0-26-gb404fb9-2.zip"),
             os.path.join(tools_dir, "xtensa", "1.20.0-26-gb404fb9-2"))
    except Exception as e:
        print(e)

    if upload:
        print("start_upload_firmware")
        bin_path = os.path.join(esp_core_lib_dir,".pio", "build", env_name, "firmware.bin")

        def recvall(sock):
            END_TOKEN = '\n'.encode('utf-8')
            BUFF_SIZE = 4096 # 4 KiB
            data = b''
            while True:
                part = sock.recv(BUFF_SIZE)
                data += part
                #print(len(part))
                if part.endswith(END_TOKEN):
                    break
            return data[:-(len(END_TOKEN))]

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((APP_HOST, APP_PORT))
        except Exception as e:
            print("mavespota error:-1: can't connect to server on " + APP_HOST + ", port " + str(APP_PORT) + ". Please, install and run NanopixGroundStation program. More info on https://minibot.tech")
            exit(1)

        json_data = None
        if json_data is None:
            try:
                json_data = u'{ "firmware" : "' + bin_path + u'" }\n'
            except Exception as e:
                print("can't setup firmware path with default decoding: " + str(e))
                json_data = None

        if json_data is None:
            try:
                json_data = u'{ "firmware" : "' + bin_path.decode('cp1251') + u'" }\n'
            except Exception as e:
                print("can't setup firmware path with cp1251 decoding: " + str(e))
                json_data = None

        if json_data is None:
            try:
                json_data = u'{ "firmware" : "' + bin_path.decode('cp866') + u'" }\n'
            except Exception as e:
                print("can't setup firmware path with cp866 decoding: " + str(e))
                json_data = None

        if json_data is None:
            try:
                json_data = u'{ "firmware" : "' + bin_path.decode('utf-8') + u'" }\n'
            except Exception as e:
                print("can't setup firmware path with utf-8 decoding: " + str(e))
                json_data = None

        message = json_data.encode("utf-8")
        s.sendall(message)
        progress_percents = 0
        while True:
            try:
                data = recvall(s)
                #print("rx data len: %s" % len(data) )
                instr = data.decode('utf-8')
            except Exception as e:
                print("mavespota error:-2: " + str(e))
                exit(1)

            if len(instr) > 0:
                try:
                    json_data = json.loads(data)
                except Exception as e:
                    print("mavespota exception: " + str(e))
                    print('Received json data: ', json_data)
                if "progress" in json_data:
                    progress_value = json_data["progress"]
                    if progress_value > progress_percents:
                        print(".*# " + str(int(progress_value)) + "%\n\r")
                        progress_percents = progress_value
                    sys.stdout.flush()
                if "result_code" in json_data:
                    print("espmavota result code: " + str(json_data["result_code"]))
                    if json_data["result_code"] == 1:
                        print("upload_script_finished")
                        print("COMPLETED!")
                        exit(0)
                    else:
                        print("ERROR!")
                        exit(1)
                if "error" in json_data:
                    err_code = json_data["error_code"]
                    print("error code: " + str())
                    if err_code == 3:
                        print("Reboot Nanopix power, please")
                    if err_code == 4:
                        print("Not enough size on device!")
                    if err_code == 5:
                        print("Device disconnected while flashing!")
                    if err_code == 6:
                        print("Failed to connect to Nanopix, check your Wi-Fi or Radio connection!")
                    if err_code == 7:
                        print("Cancelled by user!")
                    sys.stdout.flush()
                    exit(1)
