/*
    Copyright (C) 2019 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/
#pragma once

#include "stdint.h"
#include "PacketsFifo.h"
#include "PacketsFifo.cpp"

namespace NanopixCore {

class EspNowSlave {
    EspNowSlave();
    static const int MAX_DELIVERY_TIME_MS = 10;
    static const int PAIRING_STAGE_MS = 1500;
    static const int PAIRING_STAGE_FREQ_HZ = 20;
public:
    static EspNowSlave * getInstance();
    bool initializeConnection();
    void setOnReceivedCallback(void (*packetReceivedCallback)(uint8_t * data, uint8_t len));
    void addDataToSend(uint8_t * data, size_t length);
    void flush();
private:
    bool _trySend(uint8_t * data, size_t length);
    friend void espNowReceivedCallback(uint8_t *mac, uint8_t *data, uint8_t len);
    friend void espNowSentCallback(uint8_t* mac, uint8_t sendStatus);
    void (*_packetReceivedCallback)(uint8_t * data, uint8_t len);
    static EspNowSlave * _instance;
    volatile bool _msg_sent;
    uint32_t _last_msg_sent_ts = 0;
    uint8_t _controller_mac[6];
    volatile bool _pairing_reply_received;
    
    static const int ESP_NOW_TX_PACKETS_BUFFER_SIZE = 8;
    PacketsFifo<ESP_NOW_TX_PACKETS_BUFFER_SIZE> _packets_tx_fifo;

    uint8_t _server_mac[6] = {0xF2, 0xE2, 0x99, 0xA6, 0x23, 0xC3}; // TODO: load it from EEPROM (last connected one)
};

}