/*
    Copyright (C) 2020 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#include "stdint.h"
#include "IUpdater.h"

#ifndef ESP_UPDATER_H
#define ESP_UPDATER_H

namespace NanopixCore {

class EspUpdater : public IUpdater {
  static const int FIRMWARE_MD5_LEN = 16;
  static const int MAX_RETRY_WRITE_COUNT = 10;

public:
  static EspUpdater & getInstance();
  bool begin(uint32_t firmware_size, uint8_t * md5_bytes);
  bool isFwSizeValid(int32_t firmware_size);
  bool isBufferCompleted();
  void setBuffer(FwBuffer * temp_buffer);
  IUpdater::UpdateStatus getStatus() const;
  IUpdater::HandleResult handle();
  void reset();
  void end();
  int getLastFatalCode();
  IUpdater::HandleResult getLastHandleResult();
  static void md5BytesToHexString(uint8_t * input_bytes, char * output_string); // input 16 bytes, output 32 chars
private:
  EspUpdater();

  FwBuffer * _temp_buffer;
  static EspUpdater * _instance;
  uint32_t _buffer_processed_len;
  bool _is_started;
  int _retry_write_counter;
  int _custom_fatal_code;
  HandleResult _last_handle_result;
  UpdateStatus _status;
};

}

#endif
