/*
    Copyright (C) 2020 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#pragma once

#include "stdint.h"
#include "FwBuffer.h"

namespace NanopixCore {

class IUpdater {
public:
  enum UpdateStatus { NOT_STARTED = 0, IN_PROCESS, FINISHED, COMPLETED, FAILED };
  enum CommonFatalCodes { WAITING_TIMEOUT = 240, NO_NEW_DATA_TIMEOUT, WRONG_PACKET };
  enum HandleResult { HANDLE_COMPLETED = 0, WAITING, NO_NEW_DATA, 
                      ERROR_RETRY_BLOCK, FATAL_ERROR};
  virtual bool begin(uint32_t firmware_size, uint8_t * md5) = 0;
  virtual bool isFwSizeValid(int32_t firmware_size) = 0;
  virtual bool isBufferCompleted() = 0;
  virtual void setBuffer(FwBuffer * temp_buffer) = 0;
  virtual HandleResult handle() = 0;
  virtual UpdateStatus getStatus() const = 0;
  virtual void end() = 0;
  virtual void reset() = 0;
  virtual int getLastFatalCode() = 0;
  virtual HandleResult getLastHandleResult() = 0;
};

}

