/*
    Copyright (C) 2019 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/
#pragma once

#include "stdint.h"
#include "stdlib.h"

namespace NanopixCore {

struct PacketInfo {
  static const int MAX_PACKET_SIZE = 256;
  uint8_t data[MAX_PACKET_SIZE];
  size_t length;
};

template <size_t MaxSize>
class PacketsFifo {
    
public:
    PacketsFifo();

    bool pushPacket(uint8_t * data, size_t length);
    bool peekPacket(uint8_t ** data, size_t & length);
    bool pullPacket(uint8_t ** data, size_t & length);
    size_t count() const;

private:
    PacketInfo _packets[MaxSize];
    int _first_packet_id;
    size_t _packets_count;
};

}