/*
    Copyright (C) 2017 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#ifndef STM32_OTA_H
#define STM32_OTA_H

#include "stdint.h"
#include "SerialLink.h"
#include <MD5Builder.h>

namespace NanopixCore {

class STM32_OTA {
public:
   enum BootloaderAckStatus { STATUS_UNINITIALIZED = -1, BOOTLOADER_ACK_FAIL = 0, 
                              BOOTLOADER_ACK_OK, BOOTLOADER_ACK_TIMEOUT, WRONG_PACKET };
private:
  static const int FIRMWARE_MD5_LEN = 16;

  bool isOTA_Update;
  SerialLink serial_link;

  bool startAddrIsInit;
  uint8_t startAddrHighMSB, startAddrHighLSB;
  
  uint8_t _firmware_md5[FIRMWARE_MD5_LEN];
  bool _is_md5_set;
  uint16_t _current_addr;
  uint32_t _ota_fw_size;
  MD5Builder _md5;
  static STM32_OTA * _instance;
  uint32_t _flashed_bytes_count;
  bool _mavlink_mode;

  BootloaderAckStatus waitBootloaderAck(int timeout = 100000);
  void _sendAddress(uint8_t addrMSB, uint8_t addrLSB);
  void _parseSettingsRecord(uint8_t* packet_data, size_t & out_flashed_bytes);
  STM32_OTA();
public:
  static STM32_OTA & getInstance();

  static const int ACK_TOKEN_BYTE = 0x79;

  void startUpdate();
  void finishUpdate();
  
  BootloaderAckStatus write(uint8_t* packet_data, size_t data_size, size_t & out_flashed_bytes);
  BootloaderAckStatus verify(uint8_t* packet_data);
  
  BootloaderAckStatus enterToBOOT();
  BootloaderAckStatus globalErase(); // for STM32F1
  BootloaderAckStatus globalEraseEx(); // for STM32F3
  BootloaderAckStatus setReadoutUnprotect();
  BootloaderAckStatus setWriteUnprotect();

  uint32_t progress(); // returns current address

  bool isOTA_Updating();

  bool begin(uint32_t ota_fw_size = 1, bool mavlink_mode = false);

  bool isFinished();

  void setMD5Bytes(uint8_t * md5_bytes); // 16 bytes len

  bool end(); // check md5 and return result

  void reset();

};

}

#endif

