/*
    Copyright (C) 2020 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#include "stdint.h"
#include "IUpdater.h"

#ifndef STM_UPDATER_H
#define STM_UPDATER_H

namespace NanopixCore {

class StmUpdater : public IUpdater {
  static const int FIRMWARE_MD5_LEN = 16;
  static const int NO_NEW_DATA_TIMEOUT_MS = 2000;
  static const int MAX_RETRY_WRITE_COUNT = 10;
  static const int MAX_RETRY_BOOT_COUNT = 10;
  static const int MAX_RETRY_ERASE_COUNT = 10;
  enum CustomFatalCode { NO_ERROR = 0, ENTER_TO_BOOT_FAILED, ERASE_FAILED, WRITE_FAILED };
public:
  static StmUpdater & getInstance();
  bool begin(uint32_t firmware_size, uint8_t * md5_bytes);
  bool isFwSizeValid(int32_t firmware_size);
  bool isBufferCompleted();
  void setBuffer(FwBuffer * temp_buffer);
  UpdateStatus getStatus() const;
  void end();
  HandleResult handle();
  int getLastFatalCode();
  HandleResult getLastHandleResult();
  void reset();
private:
  StmUpdater();
  bool _tryEnterToBoot();
  void _tryProcessBufferData();
  void _setFatalResult(int fatal_code);
  int _custom_fatal_code;
  FwBuffer * _temp_buffer;
  static StmUpdater * _instance;
  uint32_t _buffer_processed_len;
  bool _boot_entered;
  int _retry_write_counter;
  int _retry_boot_counter;
  int _retry_erase_counter;
  int _custom_error_code;
  uint32_t _last_new_data_ts, _start_waiting_ts;
  HandleResult _last_handle_result;
  UpdateStatus _status;
};

}

#endif
