/*
Copyright (C) 2017 Nanopix
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
* Neither the name of Nanopix nor the names
of its contributors may be used to endorse or promote products
derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

#ifndef UDP_PACKET_SENDER_H
#define UDP_PACKET_SENDER_H

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include "ESPAsyncUDP.h"



namespace NanopixCore {

#define MAX_CONTROLLERS 4

enum SenderSourceType { ResendFromFC = 0, OwnMessage };

class UdpPacketSender {
public:

    /* =========================================== */
    /* =============== Constructors ============== */
    /* =========================================== */

    UdpPacketSender();

    int initialize(AsyncUDP * udp);

    /* =========================================== */
    /* ================ Operations =============== */
    /* =========================================== */

    int registerController(IPAddress addr);

    int sendToConnected(uint8_t * data, int size, SenderSourceType sourceType = SenderSourceType::ResendFromFC) const;
    int sendMulticast(uint8_t * data, int size, SenderSourceType sourceType = SenderSourceType::ResendFromFC) const;

    /* =========================================== */
    /* ================ Accessors ================ */
    /* =========================================== */

    bool hasController(IPAddress addr) const;
    bool isMainController(IPAddress addr) const;
    int connectedCount() const;
private:

    int resolvePortBySourceType(SenderSourceType sourceType) const;

    static const int CONTROLLER_UDP_PORT = 14550; // listener for resended packets
    static const int ESP_LISTENER_UDP_PORT = 5605; // listener for ESP own (internal, like wifi auth) packets

    IPAddress controllerIP[MAX_CONTROLLERS];
    IPAddress multicastIP;
    IPAddress localIP;

    AsyncUDP * udp;

    int controllersCount = 0;
public:
    IPAddress ip;
    IPAddress gateway;
    IPAddress subnet;
};

}

#endif
