/*
    Copyright (C) 2017 Nanopix
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions
    are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in
      the documentation and/or other materials provided with the 
      distribution.
    * Neither the name of Nanopix nor the names
      of its contributors may be used to endorse or promote products
      derived from this software without specific prior written
      permission.
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
    OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
    AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
    OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
    SUCH DAMAGE.
*/

#ifndef SETTINGS_MANAGER_H
#define SETTINGS_MANAGER_H

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <EEPROM.h>
#include "mavlink/common/mavlink.h"
#include "UdpPacketSender.h"
#include "nanopix_config.h"

namespace NanopixCore {

#define ESP_SOFTWARE_VERSION 2.0
#define ESP_SOFTWARE_VERSION_ID "esp_soft_vers"

#define ESP_WIFI_CHANNEL_ID "esp_wifi_chan"

class SettingsManager {
public:
  
  enum BOOT_MODES {TRY_ESPNOW_BOOT_MODE = 1, UPGRADE_BOOT_MODE, WIFI_BOOT_MODE, First = TRY_ESPNOW_BOOT_MODE, Last = WIFI_BOOT_MODE};
  
private:
  static const byte START_TOKEN_BYTE1 = 0xF5;
  static const byte START_TOKEN_BYTE2 = 0x7E;
  
  #define AUTH_DATA_ADDR (2)
  #define MAX_SSID_BYTES (30)
  #define MAX_PASS_BYTES (30)


  #define WIFI_CHANNEL_ADDR (AUTH_DATA_ADDR + MAX_SSID_BYTES + MAX_PASS_BYTES)
  #define WIFI_CHANNEL_BYTES (1)
  
  #define BOOT_MODE_ADDR (WIFI_CHANNEL_ADDR + WIFI_CHANNEL_BYTES)
  #define BOOT_MODE_BYTES (1)
  
  #define DEFAULT_PASSWORD "12345678"
    
  WiFiUDP espUDP;
  unsigned int localESP_UdpPort = 5610;

  uint16_t EEPROM_SIZE_BYTES = 512;

  mavlink_message_t rxMessage, txMessage;
  mavlink_status_t mavStatus;
  mavlink_statustext_t statusTextMsg;
  uint8_t buffer[256];
  UdpPacketSender * udpSender;
  uint8_t wifiChannel;
  bool _udpStarted;

  static SettingsManager * instance;
  
  SettingsManager();
  
  public:
  
    String SSID, PASS;
    
    static SettingsManager & getInstance();
	~SettingsManager();
    void initAll(UdpPacketSender * udpSender);

    void startUdp();

    void parseMavPackets();

    bool sendParam(char* strId);
    
    void resetSSID_PassToDefault();
	
    bool isDefaultPasswordNow();
	
    void sendSSID_PASS();

    void sendWifiChannel();
	
    String substringToToken(String src, int start, char token, int& tokenPos);

    void sendString(const char* str);
	
    void sendString(String str);

    uint8_t getWifiChannel();

    void loadWifiChannel();

    void saveWifiChannel();
	
	BOOT_MODES getNextBootMode();

    void setNextBootMode(BOOT_MODES bootMode);

    bool isEepromInitialized();

    void initEeprom();
    
    void generateRandomSSID(String& ssid);

    void saveSSID_Pass(String ssid, String pass);

    void loadSSID_Pass(String& ssid, String& pass);
};

}

#endif
