/*
Copyright (C) 2018 Nanopix
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
* Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in
the documentation and/or other materials provided with the
distribution.
* Neither the name of Nanopix nor the names
of its contributors may be used to endorse or promote products
derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/

#ifndef __STM_OTA_UDP_RECEIVER_H_
#define __STM_OTA_UDP_RECEIVER_H_

#include "stdint.h"

#define RX_BUFFER_SIZE 270

namespace NanopixCore {

class STM_OTA_UDP_Receiver {
    static const int RX_PACKET_TIMEOUT_MS = 2000;
public:
    static bool start(int udp_local_port, int udp_remote_port);
    static bool restart();
    static bool handle();
    static bool isOTA_Request();
    static void sendLog(const char* str);
    static void sendResponce(uint8_t val);
private:

    enum OTA_Commands { EnterBOOT = 0, StartUpload, StartVerify, GlobalErase, FinishUpdate, ReadoutUnprotect, WriteUnprotect };
    enum UpdateStates { UploadFlash = 0, VerifyFlash };

    static UpdateStates _current_state;
    static int _udp_remote_port, _udp_local_port;
    static uint8_t _rx_buffer[RX_BUFFER_SIZE];
    static unsigned long lastPacketTimestamp;
};

}
#endif /* __STM_OTA_UDP_RECEIVER_H_ */
