
#include "utils.h"

#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include "ESPAsyncUDP.h"

extern "C" {
#include<user_interface.h>
}

#include <SoftwareSerial.h>

SoftwareSerial * debug_serial = NULL;

void init_debug_serial()
{
#if DEBUG
  #if DEBUG_MAIN_SERIAL
    Serial.begin(DEBUG_SERIAL_BAUDRATE);
  #endif
  #if DEBUG_SW_SERIAL
    debug_serial = new SoftwareSerial(SW_SERIAL_UNUSED_PIN, DEBUG_SERIAL_TX_PIN);
    debug_serial->begin(DEBUG_SERIAL_BAUDRATE);
  #endif
#endif
}

void dbg_printf ( const char *format, ... )
{
#if DEBUG
    static char sbuf[1024];                                                     // For debug lines
    va_list varArgs;                                                            // For variable number of params

    va_start(varArgs, format);                                               // Prepare parameters
    vsnprintf(sbuf, sizeof (sbuf), format, varArgs);                       // Format the message
    va_end(varArgs );                                                         // End of using parameters
    
#if DEBUG_MAIN_SERIAL
    Serial.print(sbuf);
#endif

#if DEBUG_SW_SERIAL
    if(debug_serial != NULL)
    {
      debug_serial->print(sbuf);
    }
#endif
#endif
}

uint8_t selectOptimalWifiChannel()
{
    const int maxChannel = 12;
    int rssiPerChannel[maxChannel + 2];
    for (int i = 0; i < maxChannel + 2; i++)
        rssiPerChannel[i] = 0;

    int n = WiFi.scanNetworks();
    for (int i = 0; i < n; i++) {
        byte chan = WiFi.channel(i);
        int rssi = 255 - WiFi.RSSI(i);
/*#if DEBUG
        Serial.print("SSID : ");
        Serial.println(WiFi.SSID(i));
        Serial.print("Channel : ");
        Serial.print(chan);
        Serial.print(" RSSI : ");
        Serial.println(rssi);
#endif*/

        if (chan > maxChannel)
            chan = maxChannel;
        if (chan < 1)
            chan = 1;

        rssiPerChannel[chan - 1] += rssi / 2;
        rssiPerChannel[chan] += rssi;
        rssiPerChannel[chan + 1] += rssi / 2;
    }

/*#if DEBUG
    for (int i = 0; i < maxChannel + 2; i++) {
        Serial.print(rssiPerChannel[i]);
        Serial.print(" ");
    }
#endif*/

    uint8_t min = 255;
    uint8_t minIndex = 0;
    for (int i = 1; i < maxChannel + 1; i++) {
        int sum = rssiPerChannel[i - 1] + rssiPerChannel[i] + rssiPerChannel[i + 1];
        if (sum < min) {
            min = sum;
            minIndex = i;
        }
    }
    return minIndex;
}
//
//void printClientsStatus()
//{
//    IPAddress accessIP = WiFi.softAPIP();
//    Serial.print("ESP AccessPoint IP address: ");
//    Serial.println(accessIP);
//
//    unsigned char number_client;
//    struct station_info *stat_info;
//
//    struct ip_addr *IPaddress;
//    IPAddress address;
//    int i = 1;
//
//    number_client = wifi_softap_get_station_num(); // Count of stations which are connected to ESP8266 soft-AP
//    stat_info = wifi_softap_get_station_info();
//
//    Serial.print(" Total connected_client are = ");
//    Serial.println(number_client);
//
//    while (stat_info != NULL)
//    {
//        IPaddress = &stat_info->ip;
//        address = IPaddress->addr;
//
//        Serial.print("client= ");
//
//        Serial.print(i);
//        Serial.print(" ip adress is = ");
//        Serial.print((address));
//        Serial.print(" with mac adress is = ");
//
//        Serial.print(stat_info->bssid[0], HEX);
//        Serial.print(stat_info->bssid[1], HEX);
//        Serial.print(stat_info->bssid[2], HEX);
//        Serial.print(stat_info->bssid[3], HEX);
//        Serial.print(stat_info->bssid[4], HEX);
//        Serial.print(stat_info->bssid[5], HEX);
//
//        stat_info = STAILQ_NEXT(stat_info, next);
//        i++;
//        Serial.println();
//    }
//}
